@@ -35,10 +35,8 @@ void do_deletion(undo_type action)
 
 	/* When in the middle of a line, delete the current character. */
 	if (openfile->current->data[openfile->current_x] != '\0') {
-		int char_len = parse_mbchar(openfile->current->data +
-										openfile->current_x, NULL, NULL);
-		size_t line_len = strlen(openfile->current->data +
-										openfile->current_x);
+		int charlen = char_length(openfile->current->data + openfile->current_x);
+		size_t line_len = strlen(openfile->current->data + openfile->current_x);
 #ifndef NANO_TINY
 		/* If the type of action changed or the cursor moved to a different
 		 * line, create a new undo item, otherwise update the existing item. */
@@ -53,13 +51,13 @@ void do_deletion(undo_type action)
 #endif
 		/* Move the remainder of the line "in", over the current character. */
 		charmove(&openfile->current->data[openfile->current_x],
-					&openfile->current->data[openfile->current_x + char_len],
-					line_len - char_len + 1);
+					&openfile->current->data[openfile->current_x + charlen],
+					line_len - charlen + 1);
 #ifndef NANO_TINY
 		/* Adjust the mark if it is after the cursor on the current line. */
 		if (openfile->mark == openfile->current &&
 								openfile->mark_x > openfile->current_x)
-			openfile->mark_x -= char_len;
+			openfile->mark_x -= charlen;
 #endif
 	/* Otherwise, when not at end of buffer, join this line with the next. */
 	} else if (openfile->current != openfile->filebot) {
@@ -91,7 +89,7 @@ void do_deletion(undo_type action)
 		}
 #endif
 		unlink_node(joining);
-		renumber(openfile->current);
+		renumber_from(openfile->current);
 
 		/* Two lines were joined, so we need to refresh the screen. */
 		refresh_needed = TRUE;
@@ -148,12 +146,10 @@ void chop_word(bool forward)
 	/* Remember the current cursor position. */
 	linestruct *is_current = openfile->current;
 	size_t is_current_x = openfile->current_x;
-
-	/* Remember where the cutbuffer is and then make it seem blank. */
+	/* Remember where the cutbuffer is, then make it seem blank. */
 	linestruct *is_cutbuffer = cutbuffer;
-	linestruct *is_cutbottom = cutbottom;
+
 	cutbuffer = NULL;
-	cutbottom = NULL;
 
 	/* Move the cursor to a word start, to the left or to the right.
 	 * If that word is on another line and the cursor was not already
@@ -186,12 +182,13 @@ void chop_word(bool forward)
 	openfile->current_x = is_current_x;
 
 	/* Now kill the marked region and a word is gone. */
-	do_cut_text_void();
+	add_undo(CUT);
+	do_snip(FALSE, TRUE, FALSE, FALSE);
+	update_undo(CUT);
 
 	/* Discard the cut word and restore the cutbuffer. */
 	free_lines(cutbuffer);
 	cutbuffer = is_cutbuffer;
-	cutbottom = is_cutbottom;
 }
 
 /* Delete a word leftward. */
@@ -203,6 +200,8 @@ void chop_previous_word(void)
 /* Delete a word rightward. */
 void chop_next_word(void)
 {
+	openfile->mark = NULL;
+
 	if (is_cuttable(openfile->current_x > 0))
 		chop_word(FORWARD);
 }
@@ -215,28 +214,15 @@ void cut_line(void)
 	 * head of this line to the head of the next line into the cutbuffer;
 	 * otherwise, move all of the text of this line into the cutbuffer. */
 	if (openfile->current != openfile->filebot)
-		extract_buffer(&cutbuffer, &cutbottom, openfile->current, 0,
-				openfile->current->next, 0);
+		extract(openfile->current, 0, openfile->current->next, 0);
 	else
-		extract_buffer(&cutbuffer, &cutbottom, openfile->current, 0,
+		extract(openfile->current, 0,
 				openfile->current, strlen(openfile->current->data));
+
 	openfile->placewewant = 0;
 }
 
 #ifndef NANO_TINY
-/* Move all marked text from the current buffer into the cutbuffer. */
-void cut_marked(bool *right_side_up)
-{
-	linestruct *top, *bot;
-	size_t top_x, bot_x;
-
-	mark_order((const linestruct **)&top, &top_x,
-				(const linestruct **)&bot, &bot_x, right_side_up);
-
-	extract_buffer(&cutbuffer, &cutbottom, top, top_x, bot, bot_x);
-	openfile->placewewant = xplustabs();
-}
-
 /* Move all text from the cursor position until the end of this line into
  * the cutbuffer.  But when already at the end of a line, then move this
  * "newline" to the cutbuffer. */
@@ -248,35 +234,48 @@ void cut_to_eol(void)
 	 * the cutbuffer.  Otherwise, when not at the end of the buffer,
 	 * move the line separation into the cutbuffer. */
 	if (openfile->current_x < data_len)
-		extract_buffer(&cutbuffer, &cutbottom, openfile->current,
-				openfile->current_x, openfile->current, data_len);
+		extract(openfile->current, openfile->current_x,
+				openfile->current, data_len);
 	else if (openfile->current != openfile->filebot) {
-		extract_buffer(&cutbuffer, &cutbottom, openfile->current,
-				openfile->current_x, openfile->current->next, 0);
+		extract(openfile->current, openfile->current_x,
+				openfile->current->next, 0);
 		openfile->placewewant = xplustabs();
 	}
 }
 
+/* Move all marked text from the current buffer into the cutbuffer. */
+void cut_marked(bool *right_side_up)
+{
+	linestruct *top, *bot;
+	size_t top_x, bot_x;
+
+	get_region((const linestruct **)&top, &top_x,
+				(const linestruct **)&bot, &bot_x, right_side_up);
+
+	extract(top, top_x, bot, bot_x);
+
+	openfile->placewewant = xplustabs();
+}
+
 /* Move all text from the cursor position to end-of-file into the cutbuffer. */
 void cut_to_eof(void)
 {
-	extract_buffer(&cutbuffer, &cutbottom,
-				openfile->current, openfile->current_x,
+	extract(openfile->current, openfile->current_x,
 				openfile->filebot, strlen(openfile->filebot->data));
 }
 #endif /* !NANO_TINY */
 
 /* Move text from the current buffer into the cutbuffer.  If
- * copy_text is TRUE, copy the text back into the buffer afterward.
- * If cut_till_eof is TRUE, move all text from the current cursor
+ * copying is TRUE, copy the text back into the buffer afterward.
+ * If until_eof is TRUE, move all text from the current cursor
  * position to the end of the file into the cutbuffer.  If append
  * is TRUE (when zapping), always append the cut to the cutbuffer. */
-void do_cut_text(bool copy_text, bool marked, bool cut_till_eof, bool append)
+void do_snip(bool copying, bool marked, bool until_eof, bool append)
 {
 #ifndef NANO_TINY
-	linestruct *cb_save = NULL;
+	linestruct *was_bottom = NULL;
 		/* The current end of the cutbuffer, before we add text to it. */
-	size_t cb_save_len = 0;
+	size_t botlen = 0;
 		/* The length of the string at the current end of the cutbuffer,
 		 * before we add text to it. */
 	bool using_magicline = !ISSET(NO_NEWLINES);
@@ -284,51 +283,52 @@ void do_cut_text(bool copy_text, bool marked, bool cut_till_eof, bool append)
 	bool right_side_up = TRUE;
 		/* There *is* no region, *or* it is marked forward. */
 #endif
+	static bool precedent = FALSE;
+		/* Whether the previous operation was a copying operation. */
 
 	/* If cuts were not continuous, or when cutting a region, clear the slate. */
-	if (!append && (!keep_cutbuffer || marked || cut_till_eof)) {
+	if ((!keep_cutbuffer || marked || until_eof || copying != precedent) &&
+				!append) {
 		free_lines(cutbuffer);
 		cutbuffer = NULL;
-		/* After a line cut, future line cuts should add to the cutbuffer. */
-		keep_cutbuffer = !marked && !cut_till_eof;
 	}
 
+	/* After a line operation, future ones should add to the cutbuffer. */
+	keep_cutbuffer = !marked && !until_eof;
+	precedent = copying;
+
 #ifndef NANO_TINY
-	if (copy_text) {
+	if (copying) {
 		/* If the cutbuffer isn't empty, remember where it currently ends. */
 		if (cutbuffer != NULL) {
-			cb_save = cutbottom;
-			cb_save_len = strlen(cutbottom->data);
+			was_bottom = cutbottom;
+			botlen = strlen(cutbottom->data);
 		}
 		/* Don't add a magic line when moving text to the cutbuffer. */
 		SET(NO_NEWLINES);
 	}
 
-	if (cut_till_eof) {
-		/* Move all text up to the end of the file into the cutbuffer. */
+	/* Now move the relevant piece of text into the cutbuffer. */
+	if (until_eof)
 		cut_to_eof();
-	} else if (openfile->mark) {
-		/* Move the marked text to the cutbuffer, and turn the mark off. */
+	else if (openfile->mark) {
 		cut_marked(&right_side_up);
 		openfile->mark = NULL;
 	} else if (ISSET(CUT_FROM_CURSOR))
-		/* Move all text up to the end of the line into the cutbuffer. */
 		cut_to_eol();
 	else
 #endif
-		/* Move the entire line into the cutbuffer. */
 		cut_line();
 
 #ifndef NANO_TINY
-	if (copy_text) {
-		/* Copy the text that is in the cutbuffer (starting at its saved end,
-		 * if there is one) back into the current buffer.  This effectively
-		 * uncuts the text we just cut. */
+	if (copying) {
+		/* Copy the text that was put into the cutbuffer back into the current
+		 * file buffer, so that in the end nothing has been deleted. */
 		if (cutbuffer != NULL) {
-			if (cb_save != NULL) {
-				cb_save->data += cb_save_len;
-				copy_from_buffer(cb_save);
-				cb_save->data -= cb_save_len;
+			if (was_bottom != NULL) {
+				was_bottom->data += botlen;
+				copy_from_buffer(was_bottom);
+				was_bottom->data -= botlen;
 			} else
 				copy_from_buffer(cutbuffer);
 
@@ -341,9 +341,9 @@ void do_cut_text(bool copy_text, bool marked, bool cut_till_eof, bool append)
 		if (using_magicline)
 			UNSET(NO_NEWLINES);
 	} else
-#endif /* !NANO_TINY */
+#endif
+		set_modified();
 
-	set_modified();
 	refresh_needed = TRUE;
 }
 
@@ -372,7 +372,7 @@ bool is_cuttable(bool test_cliff)
 }
 
 /* Move text from the current buffer into the cutbuffer. */
-void do_cut_text_void(void)
+void cut_text(void)
 {
 #ifndef NANO_TINY
 	if (!is_cuttable(ISSET(CUT_FROM_CURSOR) && openfile->mark == NULL))
@@ -380,25 +380,27 @@ void do_cut_text_void(void)
 
 	/* Only add a new undo item when the current item is not a CUT or when
 	 * the current cut is not contiguous with the previous cutting. */
-	if (openfile->last_action != CUT || openfile->current_undo == NULL ||
-			openfile->current_undo->mark_begin_lineno != openfile->current->lineno ||
-			!keep_cutbuffer)
+	if (openfile->last_action != CUT || !keep_cutbuffer) {
+		keep_cutbuffer = FALSE;
 		add_undo(CUT);
-	do_cut_text(FALSE, openfile->mark != NULL, FALSE, FALSE);
+	}
+
+	do_snip(FALSE, openfile->mark != NULL, FALSE, FALSE);
+
 	update_undo(CUT);
 #else
 	if (is_cuttable(FALSE))
-		do_cut_text(FALSE, FALSE, FALSE, FALSE);
+		do_snip(FALSE, FALSE, FALSE, FALSE);
 #endif
+	wipe_statusbar();
 }
 
 #ifndef NANO_TINY
 /* Move text from the current buffer into the cutbuffer, and copy it
  * back into the buffer afterward.  If the mark is set or the cursor
  * was moved, blow away previous contents of the cutbuffer. */
-void do_copy_text(void)
+void copy_text(void)
 {
-	static linestruct *next_contiguous_line = NULL;
 	bool mark_is_set = (openfile->mark != NULL);
 
 	/* Remember the current viewport and cursor position. */
@@ -407,25 +409,22 @@ void do_copy_text(void)
 	ssize_t is_current_lineno = openfile->current->lineno;
 	size_t is_current_x = openfile->current_x;
 
-	if (mark_is_set || openfile->current != next_contiguous_line)
-		keep_cutbuffer = FALSE;
-
-	do_cut_text(TRUE, mark_is_set, FALSE, FALSE);
-
-	/* If the mark was set, blow away the cutbuffer on the next copy. */
-	next_contiguous_line = (mark_is_set ? NULL : openfile->current);
+	do_snip(TRUE, mark_is_set, FALSE, FALSE);
 
 	/* If the mark was set, restore the viewport and cursor position. */
 	if (mark_is_set) {
-		openfile->edittop = fsfromline(is_edittop_lineno);
+		openfile->edittop = line_from_number(is_edittop_lineno);
 		openfile->firstcolumn = is_firstcolumn;
-		openfile->current = fsfromline(is_current_lineno);
+		openfile->current = line_from_number(is_current_lineno);
 		openfile->current_x = is_current_x;
-	}
+	} else
+		focusing = FALSE;
+
+	openfile->last_action = OTHER;
 }
 
 /* Cut from the current cursor position to the end of the file. */
-void do_cut_till_eof(void)
+void cut_till_eof(void)
 {
 	if ((openfile->current == openfile->filebot && openfile->current->data[0] == '\0') ||
 				(!ISSET(NO_NEWLINES) && openfile->current->next == openfile->filebot &&
@@ -435,8 +434,9 @@ void do_cut_till_eof(void)
 	}
 
 	add_undo(CUT_TO_EOF);
-	do_cut_text(FALSE, FALSE, TRUE, FALSE);
+	do_snip(FALSE, FALSE, TRUE, FALSE);
 	update_undo(CUT_TO_EOF);
+	wipe_statusbar();
 }
 
 /* Erase text (current line or marked region), sending it into oblivion. */
@@ -444,33 +444,29 @@ void zap_text(void)
 {
 	/* Remember the current cutbuffer so it can be restored after the zap. */
 	linestruct *was_cutbuffer = cutbuffer;
-	linestruct *was_cutbottom = cutbottom;
 
 	if (!is_cuttable(ISSET(CUT_FROM_CURSOR) && openfile->mark == NULL))
 		return;
 
 	/* Add a new undo item only when the current item is not a ZAP or when
 	 * the current zap is not contiguous with the previous zapping. */
-	if (openfile->last_action != ZAP || openfile->current_undo == NULL ||
-			openfile->current_undo->mark_begin_lineno != openfile->current->lineno ||
-			openfile->current_undo->xflags & (MARK_WAS_SET|WAS_MARKED_FORWARD))
+	if (openfile->last_action != ZAP || !keep_cutbuffer)
 		add_undo(ZAP);
 
 	/* Use the cutbuffer from the ZAP undo item, so the cut can be undone. */
 	cutbuffer = openfile->current_undo->cutbuffer;
-	cutbottom = openfile->current_undo->cutbottom;
 
-	do_cut_text(FALSE, openfile->mark != NULL, FALSE, TRUE);
+	do_snip(FALSE, openfile->mark != NULL, FALSE, TRUE);
 
 	update_undo(ZAP);
+	wipe_statusbar();
 
 	cutbuffer = was_cutbuffer;
-	cutbottom = was_cutbottom;
 }
 #endif /* !NANO_TINY */
 
 /* Copy text from the cutbuffer into the current buffer. */
-void do_uncut_text(void)
+void paste_text(void)
 {
 	ssize_t was_lineno = openfile->current->lineno;
 		/* The line number where we started the paste. */
@@ -505,5 +501,6 @@ void do_uncut_text(void)
 	openfile->placewewant = xplustabs();
 
 	set_modified();
+	wipe_statusbar();
 	refresh_needed = TRUE;
 }
